/**---------------------------------------------------------------------------
 ** 
 ** time.h -- 
 ** 
 ** Author          : Pierre Jaccard
 ** Created On      : 1999/07/15 16:12:56
 ** Last Modified By: Pierre Jaccard
 ** Last Modified On: 1999/07/16 21:18:02
 ** Update Count    : 5
 ** Directory       : /home/pego/pcd1/codas3c/gfi/src/libs/misc/
 ** Version         : 0.0
 ** Status          : Unknown
 ** ---------------------------------------------------------------------- ** 
 ** DESCRIPTION: 
 ** 
 **    See associated file time.c
 ** 
 ** ---------------------------------------------------------------------- ** 
 ** REVISIONS: 
 ** ---------------------------------------------------------------------- ** 
 ** CHANGES: 
 **------------------------------------------------------------------------**/

#ifndef gfilib_misc_time_already_included
#define gfilib_misc_time_already_included

#include "geninc.h"
#include "ioserv.h"
	 
#if PROTOTYPE_ALLOWED

int ymdhms_to_bad(YMDHMS_TIME_TYPE *time);
int ymdhms_to_min(YMDHMS_TIME_TYPE *t);
int ymdhms_to_max(YMDHMS_TIME_TYPE *t);
int ymdhms_to_str(char *str, YMDHMS_TIME_TYPE *t, int hs);
int yd_to_ymdhms_str(char *str, double yd, int year_base);
int time_range_to_str(char *str, YMDHMS_TIME_TYPE *t1, YMDHMS_TIME_TYPE *t2);
int str_to_ymdhms(YMDHMS_TIME_TYPE *time, char *str);
int time_inside_range(YMDHMS_TIME_TYPE *t1, YMDHMS_TIME_TYPE *t2,
											YMDHMS_TIME_TYPE *t);
int fget_ymdhmh_time(FILE *fp, YMDHMS_TIME_TYPE *t);

#else

int ymdhms_to_bad();
int ymdhms_to_min();
int ymdhms_to_max();
int ymdhms_to_str();
int yd_to_ymdhms_str();
int time_range_to_str();
int str_to_ymdhms();
int time_inside_range();
int fget_ymdhmh_time();

#endif /* PROTOTYPE_ALLOWED */


#endif /* gfilib_misc_time_already_included */
